@@ -6,15 +6,15 @@ import asyncio
 from asyncio import Lock, Semaphore
 from dataclasses import dataclass
 from enum import IntEnum
-import json
 from json import JSONDecodeError
 import logging
 from typing import Any, cast
 
 from aiohttp import ClientConnectorError, ClientSession, ClientTimeout
 from aiohttp.client_reqrep import ClientResponse
+from packaging.version import Version
 
-from .common import OperationMode, get_system_zone_id
+from .common import OperationMode, get_system_zone_id, json_dumps
 from .const import (
     API_ACS_MAX_TEMP,
     API_ACS_MIN_TEMP,
@@ -57,10 +57,16 @@ from .const import (
     DEFAULT_SYSTEM_ID,
     HTTP_CALL_TIMEOUT,
     HTTP_MAX_REQUESTS,
+    HTTP_QUIRK_VERSION,
     RAW_DEMO,
     RAW_DHW,
+    RAW_HEADERS,
+    RAW_HTTP,
     RAW_HVAC,
     RAW_INTEGRATION,
+    RAW_QUIRKS,
+    RAW_REASON,
+    RAW_STATUS,
     RAW_SYSTEMS,
     RAW_VERSION,
     RAW_WEBSERVER,
@@ -82,6 +88,7 @@ from .exceptions import (
     ZoneOutOfRange,
 )
 from .hotwater import HotWater
+from .http import AirzoneHttp
 from .system import System
 from .webserver import WebServer
 from .zone import Zone
@@ -105,6 +112,7 @@ class ConnectionOptions:
     host: str
     port: int = DEFAULT_PORT
     system_id: int = DEFAULT_SYSTEM_ID
+    http_quirks: bool = False
 
 
 class AirzoneLocalApi:
@@ -120,6 +128,7 @@ class AirzoneLocalApi:
             RAW_DEMO: {},
             RAW_DHW: {},
             RAW_HVAC: {},
+            RAW_HTTP: {},
             RAW_INTEGRATION: {},
             RAW_SYSTEMS: {},
             RAW_VERSION: {},
@@ -134,6 +143,8 @@ class AirzoneLocalApi:
         self.api_features_checked = False
         self.api_features_lock = Lock()
         self.hotwater: HotWater | None = None
+        self.http = AirzoneHttp()
+        self.http_quirks_needed = True
         self.options = options
         self.systems: dict[int, System] = {}
         self.version: str | None = None
@@ -172,18 +183,21 @@ class AirzoneLocalApi:
                     raise ZoneNotProvided(f"{key}: {val}")
                 raise APIError(f"{key}: {val}")
 
-    async def http_request(
+    def http_quirks_enabled(self) -> bool:
+        """API expects HTTP headers + body on the same TCP segment."""
+        return self.options.http_quirks or self.http_quirks_needed
+
+    async def aiohttp_request(
         self, method: str, path: str, data: Any | None = None
     ) -> dict[str, Any] | None:
-        """Device HTTP request."""
-        _LOGGER.debug("aiohttp request: /%s (params=%s)", path, data)
-
+        """Perform aiohttp request."""
         async with self._api_semaphore:
             try:
                 resp: ClientResponse = await self.aiohttp_session.request(
                     method,
                     f"http://{self.options.host}:{self.options.port}/{path}",
-                    data=json.dumps(data),
+                    data=json_dumps(data),
+                    headers={"Content-Type": "text/json"},
                     timeout=self._api_timeout,
                 )
             except ClientConnectorError as err:
@@ -215,6 +229,52 @@ class AirzoneLocalApi:
 
         return cast(dict[str, Any], resp_json)
 
+    async def http_quirks_request(
+        self, method: str, path: str, data: Any | None = None
+    ) -> dict[str, Any] | None:
+        """Perform http quirks request."""
+        async with self._api_semaphore:
+            resp = await self.http.request(
+                method,
+                f"http://{self.options.host}:{self.options.port}/{path}",
+                data=json_dumps(data),
+                headers={
+                    "Content-Type": "text/json",
+                },
+                timeout=HTTP_CALL_TIMEOUT,
+            )
+
+            resp_json = resp.json()
+
+        _LOGGER.debug("aiohttp response: %s", resp_json)
+        if resp.status != 200:
+            if resp_json is not None:
+                resp_err = resp_json.get(API_ERRORS)
+            else:
+                resp_err = None
+            if resp_err is not None:
+                self.handle_errors(resp_err)
+            raise APIError(f"HTTP status: {resp.status}")
+
+        if path.endswith(API_VERSION):
+            async with self._api_raw_data_lock:
+                self._api_raw_data[RAW_HTTP][RAW_HEADERS] = resp.header_map
+                self._api_raw_data[RAW_HTTP][RAW_REASON] = resp.reason
+                self._api_raw_data[RAW_HTTP][RAW_STATUS] = resp.status
+                self._api_raw_data[RAW_HTTP][RAW_VERSION] = resp.version
+
+        return cast(dict[str, Any], resp_json)
+
+    async def http_request(
+        self, method: str, path: str, data: Any | None = None
+    ) -> dict[str, Any] | None:
+        """Device HTTP request."""
+        _LOGGER.debug("http_request: /%s (params=%s)", path, data)
+
+        if self.http_quirks_enabled():
+            return await self.http_quirks_request(method, path, data)
+        return await self.aiohttp_request(method, path, data)
+
     def update_dhw(self, data: dict[str, Any]) -> None:
         """Gather Domestic Hot Water data."""
         dhw = data.get(API_DATA, {})
@@ -282,11 +342,15 @@ class AirzoneLocalApi:
     async def check_feature_version(self) -> None:
         """Check Version feature."""
         try:
-            version = await self.get_version()
-            if version is None:
+            version_data = await self.get_version()
+            if version_data is None:
                 raise APIError("check_feature_version: empty API response")
-            if API_VERSION in version:
-                self.version = version[API_VERSION]
+            version_str = version_data.get(API_VERSION)
+            if version_str is not None:
+                self.version = version_str
+                self.http_quirks_needed = Version(version_str) < HTTP_QUIRK_VERSION
+                async with self._api_raw_data_lock:
+                    self._api_raw_data[RAW_HTTP][RAW_QUIRKS] = self.http_quirks_needed
         except InvalidMethod:
             pass
 
@@ -305,11 +369,13 @@ class AirzoneLocalApi:
 
     async def check_features(self, update: bool) -> None:
         """Check Airzone API features."""
+        # Check version and toggle HTTP quirks first.
+        await self.check_feature_version()
+
         tasks = [
             asyncio.create_task(self.check_feature_webserver()),
             asyncio.create_task(self.check_feature_systems(update)),
             asyncio.create_task(self.check_feature_dhw(update)),
-            asyncio.create_task(self.check_feature_version()),
         ]
         await asyncio.gather(*tasks)
 
